function [x chisq iter alpha covar cf ]=mrqmin_bounded_R_ss_v4(alamda,ix, mx,ndata,fun,x, opts, sig,omega,R_ss,cf,lb,ub,frequency,amplitude)
% This script follows the  'mrqmin' fortran program found in Numerical
% Recipes Chapter 15.5. The program seeks to minimize the chisq
% calculation by finding a set of parameters x in an iterative manner. The
% program returns:
%   x - the parameter vector
%   chisq - the calculated sum of squares
%   iter - the number of iterations used to find x
%   alpha - the approximated Hessian matrix
%   curve - the inverse of alpha; used in analysis of errors. 

% amended 12-2-10 to include new check_bounds_v4
global h;
global BIND BINDYZ BINDXZ;
%% this section provides initialization conditions. 
iter=0; chisqdiff=1;stop=0;

if alamda < 0
   % mfit=0;
%for j=1:mx   
    %if ix(j)~=0
       % mfit=mfit+1; end   
%end
mfit=sum(ix>0);
alamda=.001;
x=check_bounds_v4(x,lb,ub,ix);
[alpha beta chisq cf ]=mrqcof_R_ss_v4(fun,sig,ndata,x,ix,mx,opts,omega,R_ss,cf,frequency,amplitude);

ochisq=chisq;
xtry=x;
end

%% Iterations start in this section
% while iter < opts(4)&& chisqdiff > opts(3); %these are stopping criteria
while (~stop && iter <= opts(4)); 
    
covar=alpha+(alamda*eye(mfit));%adjusts alpha by a factor of (1+alambda) on the diagonal
dx=beta;


%% this section provides two ways to solve the Normal Equations. 
% gaussj uses the gauss jordan method for matrix inversion. gaussjmatlab
% uses matlab's built in functions to do the same calculation. 
%[inverse dx]=gaussj(covar,mfit,mfit,dx',1,1);
[dx]=gaussjmatlabv2(alpha,dx,alamda);%calculates the 'step' in parameter space
%% This section is to be used at the end of the program.
if alamda == 0
    [covar]=covsrt(covar,mfit,mx,ix,mfit);
    [alpha]=covsrt(alpha,mfit,mx,ix,mfit);%spread out alpha to its full size too
    return
end
%% This section updates the initial parameter vector by the step calculated
%through the matlab program. 
j=0;
for l=1:mx
    if ix(l) ~= 0
        j=j+1;
        xtry(l)=x(l)+ dx(j);%calculate the new parameter vector
    end       
%% Check bounds and create new xtry
xtry=check_bounds_v4(xtry, lb, ub,ix);
end

        %if BIND == 1 % BIND global variable connects Sexx and Seyy, Smuxx and Smuyy
        %while kll <= mx
            
        %end 
    %end
    
%    %BIND connects Sexx and Seyy, Smxx and Smuyy BE CAREFUL WITH BOUNDS
%    make sure that they the same as in xx
   if BIND == 1
      for k = 0:35
        xtry(4+k*8) = xtry(3+k*8); 
        xtry(7+k*8) = xtry(6+k*8);
        xtry(36*8+2) = xtry(36*8+1);
      end
   end

  if BINDYZ == 1
      for k = 0:35
        xtry(5+k*8) = xtry(4+k*8); 
        xtry(8+k*8) = xtry(7+k*8);
      end
  end
  
  if BINDXZ == 1
      for k = 0:35
        xtry(5+k*8) = xtry(3+k*8); 
        xtry(8+k*8) = xtry(6+k*8);
      end
  end
  
[covar dx chisq cf]=mrqcof_R_ss_v4(fun,sig,ndata,xtry,ix,mx,opts,omega,R_ss,cf,frequency,amplitude); 

chisqdiff=abs(chisq-ochisq);

if chisq < ochisq; %did the trial succeed?
   %disp('chisq is getting smaller')
   if ( abs(ochisq-chisq)<opts(3))
    disp(' **** Convergence in Chi-square  **** ')
	 %close(h);
    stop = 1;
    
   end
    alamda=0.1*alamda;
    ochisq=chisq;
   
       alpha=covar;
       beta=dx;
       
    for l=1:mx
    x(l)=xtry(l);%accept the new parameter vector
    end
else alamda=10*alamda;%try a bigger alamda
    chisq=ochisq;
    %disp('chisq is getting bigger')
    
    
end
%MK    
   % fprintf(' iteration # %d ',iter);
   % fprintf(' # of func calls %d ', cf);  
   % fprintf(' chisq = %d \n', chisq);
   % fprintf(' params values ');fprintf(' %d, ',x);
   % fprintf('------------------------------------------------------------------------------\n');




% % Plot the Parameters and Chisq of the Current Iteration
% figure(1)
% subplot(2,1,1)
% bar(1:mx,x)
% title('Parameter Values','fontsize',12)
% subplot(2,1,2)
% plot(iter,chisq,'--r*')
% xlabel('Iteration');
% ylabel('Chi Squared');
% title('Convergence of Chisq','fontsize',12);
% hold on

%  if ( max(abs(dx./x)) < opts(2)  &  iter > 2 ) 
% 	disp(' **** Convergence in Parameters **** ')
% 	stop = 1;
%    end
% %    if ( chisq/ndata < opts(3)  &  iter > 2 ) 
% % 	disp(' **** Convergence in Chi-square  **** ')
% % 	stop = 1;
% %    end

%    if ( max(abs(beta)) < opts(1)  &&  iter > 2 ) 
% 	disp(' **** Convergence in r.h.s. ("beta")  **** ')
%      %close(h);
% 	stop = 1;
%    end
   if ( iter == opts(4) )
	disp(' !! Maximum Number of Iterations Reached Without Convergence !!')
    %close(h);
        stop = 1;
   end
   
iter=iter+1;   
   if stop ~= 1
waitbar(iter/opts(4),h,'Fitting...');
   else
       close(h);
   end


end
%hold off
%% this section produces the alpha and covar matrices


covar=inv(alpha);

  [alpha]=covsrtv2(alpha,mfit,mx,ix,mfit);
   
   [covar]=covsrtv2(covar,mfit,mx,ix,mfit);
    